@@ -17,7 +17,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
 """
@@ -32,7 +32,7 @@ Specific symbols for parts of a name are defined:
     'f'     given (first names)
     'l'     full surname (lastname)
     'c'     callname
-    'x'     nick name if existing, otherwise first first name (common name)
+    'x'     nick name, call, or otherwise first first name (common name)
     'i'     initials of the first names
     'm'     primary surname (main)
     '0m'    primary surname prefix
@@ -68,7 +68,7 @@ LOG = logging.getLogger(".gramps.gen")
 #-------------------------------------------------------------------------
 from ..const import ARABIC_COMMA, ARABIC_SEMICOLON, GRAMPS_LOCALE as glocale
 _ = glocale.translation.sgettext
-from ..constfunc import cuni, conv_to_unicode, UNITYPE
+from ..constfunc import conv_to_unicode
 from ..lib.name import Name
 from ..lib.nameorigintype import NameOriginType
 
@@ -567,7 +567,7 @@ class NameDisplay(object):
         'f' : given      = given (first names)
         'l' : surname    = full surname (lastname)
         'c' : call       = callname
-        'x' : common     = nick name if existing, otherwise first first name (common name)
+        'x' : common     = nick name, call, otherwise first first name (common name)
         'i' : initials   = initials of the first names
         'm' : primary    = primary surname (main)
         '0m': primary[pre]= prefix primary surname (main)
@@ -601,7 +601,7 @@ class NameDisplay(object):
                                 _("suffix")),
              "c": ("raw_data[_CALL]",      "call",       
                                 _("Name|call")),
-             "x": ("(raw_data[_NICK] or raw_data[_FIRSTNAME].split(' ')[0])",
+             "x": ("(raw_data[_NICK] or raw_data[_CALL] or raw_data[_FIRSTNAME].split(' ')[0])",
                                 "common",
                                 _("Name|common")),
              "i": ("''.join([word[0] +'.' for word in ('. ' +" +
@@ -668,7 +668,7 @@ class NameDisplay(object):
         'f' : given      = given (first names)
         'l' : surname    = full surname (lastname)
         'c' : call       = callname
-        'x' : common     = nick name if existing, otherwise first first name (common name)
+        'x' : common     = nick name, call, or otherwise first first name (common name)
         'i' : initials   = initials of the first names
         'm' : primary    = primary surname (main)
         '0m': primary[pre]= prefix primary surname (main)
@@ -701,7 +701,7 @@ class NameDisplay(object):
                         _("suffix")),
              "c": ("call",       "call",       
                         _("Name|call")),
-             "x": ("(nick or first.split(' ')[0])", "common", 
+             "x": ("(nick or call or first.split(' ')[0])", "common", 
                         _("Name|common")),
              "i": ("''.join([word[0] +'.' for word in ('. ' + first).split()][1:])",
                         "initials", 
@@ -770,7 +770,7 @@ class NameDisplay(object):
         '%f' : given (first names)
         '%l' : full surname (lastname)
         '%c' : callname
-        '%x' : nick name if existing, otherwise first first name (common name)
+        '%x' : nick name, call, or otherwise first first name (common name)
         '%i' : initials of the first names
         '%m' : primary surname (main)
         '%0m': prefix primary surname (main)
@@ -816,7 +816,7 @@ class NameDisplay(object):
         return sur.get_surname()
 
     def sort_string(self, name):
-        return cuni("%-25s%-30s%s") % (self.primary_surname(name),
+        return "%-25s%-30s%s" % (self.primary_surname(name),
                                   name.first_name, name.suffix)
 
     def sorted(self, person):
@@ -1022,7 +1022,7 @@ class NameDisplay(object):
             # if in double quotes, just use % codes
             for (code, keyword) in d_keys:
                 exp, keyword, ikeyword = d[code]
-                if not isinstance(keyword, UNITYPE):
+                if not isinstance(keyword, str):
                     keyword = conv_to_unicode(keyword, "utf-8")
                 format_str = format_str.replace(keyword, "%"+ code)
                 format_str = format_str.replace(keyword.title(), "%"+ code)
