@@ -17,7 +17,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
 """
@@ -29,20 +29,15 @@ This package implements access to Gramps configuration.
 # System imports
 #
 #---------------------------------------------------------------
-from __future__ import print_function, unicode_literals
 import os
 import sys
 import time
-if sys.version_info[0] < 3:
-    import ConfigParser as configparser
-else:
-    import configparser
+import configparser
 import errno
 import copy
 import logging
 import io
 
-from ..constfunc import STRTYPE
 from ..const import GRAMPS_LOCALE as glocale
 _ = glocale.translation.gettext
 
@@ -157,7 +152,7 @@ class ConfigManager(object):
                                       use_plugins_path=False)
           # will use /tmp/Other.ini
         """
-        if isinstance(override, STRTYPE): # directory or filename
+        if isinstance(override, str): # directory or filename
             if override:
                 path, ininame = os.path.split(os.path.abspath(override))
             else:
@@ -260,10 +255,7 @@ class ConfigManager(object):
         if filename and os.path.exists(filename):
             parser = configparser.RawConfigParser()
             try: # see bugs 5356, 5490, 5591, 5651, 5718, etc.
-                if sys.version_info[0] >= 3 :
-                    parser.read(filename, encoding='utf8')
-                else:
-                    parser.readfp(io.open(filename, encoding="utf-8"))
+                parser.read(filename, encoding='utf8')
             except Exception as err:
                 msg1 = _("WARNING: could not parse file %s because %s, recreating it:\n")
                 logging.warn(msg1 % (filename, str(err)))
@@ -276,12 +268,11 @@ class ConfigManager(object):
                     self.data[name] = {}
                 for opt in parser.options(sec):
                     raw_value = parser.get(sec, opt).strip()
-                    if sys.version_info[0] >= 3:
-                        if raw_value[:2] == "u'":
-                            raw_value = raw_value[1:]
-                        elif raw_value.startswith('['):
-                            raw_value = raw_value.replace(", u'", ", '")
-                            raw_value = raw_value.replace("[u'", "['")
+                    if raw_value[:2] == "u'":
+                        raw_value = raw_value[1:]
+                    elif raw_value.startswith('['):
+                        raw_value = raw_value.replace(", u'", ", '")
+                        raw_value = raw_value.replace("[u'", "['")
                     setting = opt.lower()
                     if oldstyle:
                     ####################### Upgrade from oldstyle < 3.2
@@ -552,8 +543,8 @@ class ConfigManager(object):
         type2 = type(value2)
         if type1 == type2:
             return True
-        elif (isinstance(value1, STRTYPE) and
-              isinstance(value2, STRTYPE)):
+        elif (isinstance(value1, str) and
+              isinstance(value2, str)):
             return True
         elif (type1 in [int, float] and 
               type2 in [int, float]):
