@@ -17,7 +17,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 
 """
@@ -29,11 +29,7 @@ Provide the base classes for GRAMPS' DataView classes
 # python modules
 #
 #----------------------------------------------------------------
-import sys
-if sys.version_info[0] < 3:
-    import cPickle as pickle
-else:
-    import pickle
+import pickle
 import time
 import logging
 
@@ -55,12 +51,13 @@ from gi.repository import Pango
 #----------------------------------------------------------------
 from .pageview import PageView
 from .navigationview import NavigationView
+from ..actiongroup import ActionGroup
 from ..columnorder import ColumnOrder
 from gramps.gen.config import config
 from gramps.gen.errors import WindowActiveError
 from ..filters import SearchBar
 from ..widgets.menuitem import add_menuitem
-from gramps.gen.constfunc import UNITYPE, conv_to_unicode
+from gramps.gen.constfunc import conv_to_unicode
 from gramps.gen.const import CUSTOM_FILTERS
 from gramps.gen.utils.debug import profile
 from gramps.gen.utils.string import data_recover_msg
@@ -71,6 +68,7 @@ _ = glocale.translation.sgettext
 from ..ddtargets import DdTargets
 from ..plug.quick import create_quickreport_menu, create_web_connect_menu
 from ..utils import is_right_click
+from ..widgets.interactivesearchbox import InteractiveSearchBox
 
 #----------------------------------------------------------------
 #
@@ -142,7 +140,7 @@ class ListView(NavigationView):
         contains the interface. This containter will be inserted into
         a Gtk.Notebook page.
         """
-        self.vbox = Gtk.VBox()
+        self.vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         self.vbox.set_border_width(4)
         self.vbox.set_spacing(4)
         
@@ -151,12 +149,13 @@ class ListView(NavigationView):
         filter_box = self.search_bar.build()
 
         self.list = Gtk.TreeView()
-        self.list.set_rules_hint(True)
         self.list.set_headers_visible(True)
         self.list.set_headers_clickable(True)
         self.list.set_fixed_height_mode(True)
         self.list.connect('button-press-event', self._button_press)
         self.list.connect('key-press-event', self._key_press)
+        self.list.connect('start-interactive-search',self.open_all_nodes)
+        self.searchbox = InteractiveSearchBox(self.list)
         
         if self.drag_info():
             self.list.connect('drag_data_get', self.drag_data_get)
@@ -206,23 +205,23 @@ class ListView(NavigationView):
         
         NavigationView.define_actions(self)
 
-        self.edit_action = Gtk.ActionGroup(name=self.title + '/ChangeOrder')
+        self.edit_action = ActionGroup(name=self.title + '/ChangeOrder')
         self.edit_action.add_actions([
-                ('Add', Gtk.STOCK_ADD, _("_Add..."), "<PRIMARY>Insert", 
-                    self.ADD_MSG, self.add), 
-                ('Remove', Gtk.STOCK_REMOVE, _("_Remove"), "<PRIMARY>Delete", 
-                    self.DEL_MSG, self.remove), 
+                ('Add', 'list-add', _("_Add..."), "<PRIMARY>Insert",
+                    self.ADD_MSG, self.add),
+                ('Remove', 'list-remove', _("_Remove"), "<PRIMARY>Delete",
+                    self.DEL_MSG, self.remove),
                 ('Merge', 'gramps-merge', _('_Merge...'), None,
                     self.MERGE_MSG, self.merge),
                 ('ExportTab', None, _('Export View...'), None, None,
-                    self.export), 
+                    self.export),
                 ])
 
         self._add_action_group(self.edit_action)
 
-        self._add_action('Edit', Gtk.STOCK_EDIT, _("action|_Edit..."), 
-                         accel="<PRIMARY>Return", 
-                         tip=self.EDIT_MSG, 
+        self._add_action('Edit', 'gtk-edit', _("action|_Edit..."),
+                         accel="<PRIMARY>Return",
+                         tip=self.EDIT_MSG,
                          callback=self.edit)
         
     def build_columns(self):
@@ -247,7 +246,7 @@ class ListView(NavigationView):
 
             if col_icon is not None:
                 image = Gtk.Image()
-                image.set_from_stock(col_icon, Gtk.IconSize.MENU)
+                image.set_from_icon_name(col_icon, Gtk.IconSize.MENU)
                 image.set_tooltip_text(col_name)
                 image.show()
                 column.set_widget(image)
@@ -268,25 +267,23 @@ class ListView(NavigationView):
 
     def icon(self, column, renderer, model, iter_, col_num):
         '''
-        Set the stock icon property of the cell renderer.  We use a cell data
+        Set the icon-name property of the cell renderer.  We use a cell data
         function because there is a problem returning None from a model.
         '''
-        stock_id = model.get_value(iter_, col_num)
-        if stock_id == '':
-            stock_id = None
-        renderer.set_property('stock_id', stock_id)
+        icon_name = model.get_value(iter_, col_num)
+        if icon_name == '':
+            icon_name = None
+        renderer.set_property('icon-name', icon_name)
 
     def foreground_color(self, column, renderer, model, iter_, data=None):
         '''
         Set the foreground color of the cell renderer.  We use a cell data
-        function because we don't want to set the color of untagged rows.
+        function because there is a problem returning None from a model.
         '''
         fg_color = model.get_value(iter_, model.color_column())
-        #for color errors, typically color column is badly set
-        if fg_color:
-            renderer.set_property('foreground', fg_color)
-        else:
-            LOG.debug('Bad color set: ' + str(fg_color))
+        if fg_color == '':
+            fg_color = None
+        renderer.set_property('foreground', fg_color)
 
     def set_active(self):
         """
@@ -449,7 +446,8 @@ class ListView(NavigationView):
             WarningDialog(
                 _("Could Not Set a Bookmark"), 
                 _("A bookmark could not be set because "
-                  "nothing was selected."))
+                  "nothing was selected."),
+                parent=self.uistate.window)
 
     ####################################################################
     # 
@@ -474,7 +472,7 @@ class ListView(NavigationView):
         return None
 
     def drag_begin(self, widget, context):
-        widget.drag_source_set_icon_stock(self.get_stock())
+        widget.drag_source_set_icon_name(self.get_stock())
         
     def drag_data_get(self, widget, context, sel_data, info, time):
         selected_ids = self.selected_handles()
@@ -539,11 +537,12 @@ class ListView(NavigationView):
         prompt = True
         if len(self.selected_handles()) > 1:
             q = QuestionDialog2(
-                _("Confirm every deletion?"),
+                _("Multiple Selection Delete"),
                 _("More than one item has been selected for deletion. "
-                  "Ask before deleting each one?"),
-                _("No"),
-                _("Yes"))
+                  "Select the option indicating how to delete the items:"),
+                _("Delete All"),
+                _("Confirm Each Delete"),
+                parent=self.uistate.window)
             prompt = not q.run()
             
         if not prompt:
@@ -565,7 +564,8 @@ class ListView(NavigationView):
                 descr = object.get_gramps_id()
                 self.uistate.set_busy_cursor(True)
                 QuestionDialog(_('Delete %s?') % descr, msg,
-                               _('_Delete Item'), query.query_response)
+                               _('_Delete Item'), query.query_response,
+                               parent=self.uistate.window)
                 self.uistate.set_busy_cursor(False)
             else:
                 query.query_response()
@@ -632,7 +632,7 @@ class ListView(NavigationView):
         if same_col:
             # activate when https://bugzilla.gnome.org/show_bug.cgi?id=684558
             # is resolved
-            if False and (Gtk.get_major_version(), Gtk.get_minor_version()) >= (3,8):
+            if False:
                 self.model.reverse_order()
             else:
                 ## GTK 3.6 rows_reordered not exposed by gi, we need to reconnect
@@ -875,10 +875,10 @@ class ListView(NavigationView):
             return False
         if self.model.get_flags() & Gtk.TreeModelFlags.LIST_ONLY:
             # Flat list
-            self._key_press_flat(obj, event)
+            return self._key_press_flat(obj, event)
         else:
             # Tree
-            self._key_press_tree(obj, event)
+            return self._key_press_tree(obj, event)
     
     def _key_press_flat(self, obj, event):
         """
@@ -888,6 +888,9 @@ class ListView(NavigationView):
         if event.keyval in (Gdk.KEY_Return, Gdk.KEY_KP_Enter):
             self.edit(obj)
             return True
+        # Custom interactive search
+        if Gdk.keyval_to_unicode(event.keyval):
+            return self.searchbox.treeview_keypress(obj, event)
         return False
 
     def _key_press_tree(self, obj, event):
@@ -896,16 +899,15 @@ class ListView(NavigationView):
         ENTER --> edit selection or open group node
         SHIFT+ENTER --> open group node and all children nodes
         """
-        if event.get_state() & Gdk.ModifierType.SHIFT_MASK:
-            if event.keyval in (Gdk.KEY_Return, Gdk.KEY_KP_Enter):
-                store, paths = self.selection.get_selected_rows()
-                if paths:
-                    iter_ = self.model.get_iter(paths[0])
-                    handle = self.model.get_handle_from_iter(iter_)
-                    if len(paths) == 1 and handle is None:
-                        return self.expand_collapse_tree_branch()
-        else:
-            if event.keyval in (Gdk.KEY_Return, Gdk.KEY_KP_Enter):
+        if (event.get_state() & Gdk.ModifierType.SHIFT_MASK and
+            event.keyval in (Gdk.KEY_Return, Gdk.KEY_KP_Enter)):
+            store, paths = self.selection.get_selected_rows()
+            if paths:
+                iter_ = self.model.get_iter(paths[0])
+                handle = self.model.get_handle_from_iter(iter_)
+                if len(paths) == 1 and handle is None:
+                    return self.expand_collapse_tree_branch()
+        elif event.keyval in (Gdk.KEY_Return, Gdk.KEY_KP_Enter):
                 store, paths = self.selection.get_selected_rows()
                 if paths:
                     iter_ = self.model.get_iter(paths[0])
@@ -915,6 +917,9 @@ class ListView(NavigationView):
                     else:
                         self.edit(obj)
                         return True
+        elif Gdk.keyval_to_unicode(event.keyval):
+            # Custom interactive search
+            return self.searchbox.treeview_keypress(obj, event)
         return False
 
     def expand_collapse_tree(self):
@@ -996,14 +1001,14 @@ class ListView(NavigationView):
             _("Export View as Spreadsheet"), 
             self.uistate.window, 
             Gtk.FileChooserAction.SAVE, 
-            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, 
-             Gtk.STOCK_SAVE, Gtk.ResponseType.OK))
+            (_('_Cancel'), Gtk.ResponseType.CANCEL,
+             _('_Save'), Gtk.ResponseType.OK))
         chooser.set_do_overwrite_confirmation(True)
 
         combobox = Gtk.ComboBoxText()
         label = Gtk.Label(label=_("Format:"))
-        label.set_alignment(1.0, 0.5)
-        box = Gtk.HBox()
+        label.set_halign(Gtk.Align.END)
+        box = Gtk.Box()
         box.pack_start(label, True, True, padding=12)
         box.pack_start(combobox, False, False, 0)
         combobox.append_text(_('CSV'))
