@@ -138,7 +138,7 @@ class CitationBaseModel(object):
         tag_handle = data[0]
         cached, tag_color = self.get_cached_value(tag_handle, "TAG_COLOR")
         if not cached:
-            tag_color = "#000000000000"
+            tag_color = ""
             tag_priority = None
             for handle in data[COLUMN_TAGS]:
                 tag = self.db.get_tag_from_handle(handle)
@@ -297,7 +297,7 @@ class CitationBaseModel(object):
         tag_handle = data[0]
         cached, tag_color = self.get_cached_value(tag_handle, "TAG_COLOR")
         if not cached:
-            tag_color = "#000000000000"
+            tag_color = ""
             tag_priority = None
             for handle in data[COLUMN2_TAGS]:
                 tag = self.db.get_tag_from_handle(handle)
