@@ -39,7 +39,6 @@ from gi.repository import Gtk
 #
 #-------------------------------------------------------------------------
 from gramps.gen.datehandler import format_time
-from gramps.gen.constfunc import cuni
 from .flatbasemodel import FlatBaseModel
 from gramps.gen.const import GRAMPS_LOCALE as glocale
 
@@ -100,19 +99,19 @@ class SourceModel(FlatBaseModel):
         return len(self.fmap)+1
 
     def column_title(self,data):
-        return cuni(data[2])
+        return str(data[2])
 
     def column_author(self,data):
-        return cuni(data[3])
+        return str(data[3])
 
     def column_abbrev(self,data):
-        return cuni(data[7])
+        return str(data[7])
 
     def column_id(self,data):
-        return cuni(data[1])
+        return str(data[1])
 
     def column_pubinfo(self,data):
-        return cuni(data[4])
+        return str(data[4])
 
     def column_private(self, data):
         if data[12]:
@@ -131,22 +130,31 @@ class SourceModel(FlatBaseModel):
         """
         Return the tag name from the given tag handle.
         """
-        return self.db.get_tag_from_handle(tag_handle).get_name()
+        cached, value = self.get_cached_value(tag_handle, "TAG_NAME")
+        if not cached:
+            value = self.db.get_tag_from_handle(tag_handle).get_name()
+            self.set_cached_value(tag_handle, "TAG_NAME", value)        
+        return value 
         
     def column_tag_color(self, data):
         """
         Return the tag color.
         """
-        tag_color = "#000000000000"
-        tag_priority = None
-        for handle in data[11]:
-            tag = self.db.get_tag_from_handle(handle)
-            if tag:
-                this_priority = tag.get_priority()
-                if tag_priority is None or this_priority < tag_priority:
-                    tag_color = tag.get_color()
-                    tag_priority = this_priority
-        return tag_color
+        tag_handle = data[0]
+        cached, value = self.get_cached_value(tag_handle, "TAG_COLOR")
+        if not cached:
+            tag_color = ""
+            tag_priority = None
+            for handle in data[11]:
+                tag = self.db.get_tag_from_handle(handle)
+                if tag:
+                    this_priority = tag.get_priority()
+                    if tag_priority is None or this_priority < tag_priority:
+                        tag_color = tag.get_color()
+                        tag_priority = this_priority
+            value = tag_color
+            self.set_cached_value(tag_handle, "TAG_COLOR", value)        
+        return value 
 
     def column_tags(self, data):
         """
