@@ -235,6 +235,8 @@ class GeoFamClose(GeoGraphyView):
         self.sort = []
         self.places_found = []
         self.place_without_coordinates = []
+        self.nbmarkers = 0
+        self.nbplaces = 0
         self.remove_all_gps()
         self.remove_all_markers()
         self.lifeway_layer.clear_ways()
@@ -248,11 +250,14 @@ class GeoFamClose(GeoGraphyView):
         if self.reffamily:
             color = self._config.get('geography.color1')
             self._createmap(self.reffamily, color, self.place_list_ref, True)
-            self.message_layer.add_message(_("Family reference : %s" % self.family_label(self.reffamily)))
+            self.message_layer.add_message(_("Family reference : %s") %
+                                           self.family_label(self.reffamily))
             if f1:
-                self.message_layer.add_message(_("The other family : %s" % self.family_label(f1)))
+                self.message_layer.add_message(_("The other family : %s") %
+                                               self.family_label(f1))
             else:
-                self.message_layer.add_message(_("The other family : %s" % _("Unknown")))
+                self.message_layer.add_message(_("The other family : %s") %
+                                               _("Unknown"))
             if self.reffamily_bookmark is None:
                 self.reffamily_bookmark = self.reffamily.get_handle()
                 self.add_bookmark_from_popup(None, self.reffamily_bookmark)
@@ -299,6 +304,9 @@ class GeoFamClose(GeoGraphyView):
         all handling of visibility is now in rebuild_trees, see that for more
         information.
         """
+        self.lifeway_layer.clear_ways()
+        if not self.dbstate.open:
+            return
         active = self.get_active()
         family = self.dbstate.db.get_family_from_handle(active)
         self.goto_handle(handle=family)
@@ -678,9 +686,7 @@ class GeoFamClose(GeoGraphyView):
                                    event, lat, lon, prevmark)
                     itemoption.append(center)
                     menu.show()
-                    menu.popup(None, None,
-                               lambda menu, data: (event.get_root_coords()[0],
-                                                   event.get_root_coords()[1], True),
+                    menu.popup(None, None, None,
                                None, event.button, event.time)
         return 0
 
@@ -709,11 +715,11 @@ class GeoFamClose(GeoGraphyView):
         Add specific entry to the preference menu.
         Must be done in the associated view.
         """
-        table = Gtk.Table(n_rows=2, n_columns=2)
-        table.set_border_width(12)
-        table.set_col_spacings(6)
-        table.set_row_spacings(6)
-        configdialog.add_text(table,
+        grid = Gtk.Grid()
+        grid.set_border_width(12)
+        grid.set_column_spacing(6)
+        grid.set_row_spacing(6)
+        configdialog.add_text(grid,
                 _('The meeting zone probability radius.\n'
                   'The colored zone is approximative.\n'
                   'The meeting zone is only shown for the reference family.\n'
@@ -721,11 +727,11 @@ class GeoFamClose(GeoGraphyView):
                   'The value 1 means about 4.6 miles or 7.5 kms.\n'
                   'The value is in tenth of degree.'),
                 1, line_wrap=False)
-        self.config_meeting_slider = configdialog.add_slider(table, 
+        self.config_meeting_slider = configdialog.add_slider(grid,
                 "", 
                 2, 'geography.maximum_meeting_zone',
                 (1, 9))
-        return _('The selection parameters'), table
+        return _('The selection parameters'), grid
 
     def config_connect(self):
         """
