@@ -32,7 +32,6 @@ __all__ = ["MonitoredCheckbox", "MonitoredEntry",
 #-------------------------------------------------------------------------
 import logging
 _LOG = logging.getLogger(".widgets.monitoredwidgets")
-import sys
 
 #-------------------------------------------------------------------------
 #
@@ -55,7 +54,6 @@ from ..autocomp import StandardCustomSelector, fill_entry
 from gramps.gen.datehandler import displayer, parser
 from gramps.gen.lib.date import Date, NextYear
 from gramps.gen.errors import ValidationError
-from gramps.gen.constfunc import cuni
 
 #-------------------------------------------------------------------------
 #
@@ -139,11 +137,7 @@ class MonitoredEntry(object):
         self.obj.connect(signal, callback, *data)
 
     def _on_change(self, obj):
-        if sys.version_info[0] < 3:
-            self.set_val(cuni(obj.get_text(), 'utf-8'))
-        else:
-            #all is unicode
-            self.set_val(obj.get_text())
+        self.set_val(obj.get_text())
         if self.changed:
             self.changed(obj)
 
@@ -151,7 +145,7 @@ class MonitoredEntry(object):
         self.obj.set_text(value)
 
     def get_value(self, value):
-        return cuni(self.obj.get_text())
+        return str(self.obj.get_text())
 
     def enable(self, value):
         self.obj.set_sensitive(value)
@@ -194,7 +188,7 @@ class MonitoredEntryIndicator(MonitoredEntry):
     
     def _on_change(self, obj):
         if not self.indicatorshown:
-            self.set_val(cuni(obj.get_text()))
+            self.set_val(str(obj.get_text()))
             if self.changed:
                 self.changed(obj)
 
@@ -367,7 +361,7 @@ class MonitoredText(object):
 
     def on_change(self, obj):
         s, e = self.buf.get_bounds()
-        self.set_val(cuni(self.buf.get_text(s, e, False)))
+        self.set_val(str(self.buf.get_text(s, e, False)))
 
 #-------------------------------------------------------------------------
 #
@@ -626,7 +620,7 @@ class MonitoredDate(object):
         self.button_obj = button
 
         image = Gtk.Image()
-        image.set_from_stock('gramps-date-edit', Gtk.IconSize.BUTTON)
+        image.set_from_icon_name('gramps-date-edit', Gtk.IconSize.BUTTON)
         self.button_obj.set_image(image)
         self.button_obj.set_relief(Gtk.ReliefStyle.NORMAL)
         self.pixmap_obj = self.button_obj.get_child()
@@ -645,7 +639,7 @@ class MonitoredDate(object):
         """
         Parse date from text entry to date object
         """
-        date = parser.parse(cuni(self.text_obj.get_text()))
+        date = parser.parse(str(self.text_obj.get_text()))
         self.date_obj.copy(date)
         
     def validate(self, widget, data):
@@ -797,7 +791,7 @@ class MonitoredComboSelectedEntry(object):
         self.set_val_list[self.active_key](self.get_value_entry())
 
     def get_value_entry(self):
-        return cuni(self.objentry.get_text())
+        return str(self.objentry.get_text())
 
     def enable(self, value):
         self.objentry.set_sensitive(value)
@@ -835,14 +829,15 @@ class MonitoredTagList(object):
                 self.tag_list.append((handle, tag.get_name()))
 
         self.all_tags = []
-        for tag in self.db.iter_tags():
+        for handle in self.db.get_tag_handles(sort_handles=True):
+            tag = self.db.get_tag_from_handle(handle)
             self.all_tags.append((tag.get_handle(), tag.get_name()))
 
         self.label = label
-        self.label.set_alignment(0, 0.5)
+        self.label.set_halign(Gtk.Align.START)
         self.label.set_ellipsize(Pango.EllipsizeMode.END)
         image = Gtk.Image()
-        image.set_from_stock('gramps-tag', Gtk.IconSize.MENU)
+        image.set_from_icon_name('gramps-tag', Gtk.IconSize.MENU)
         button.set_image (image)
         button.set_tooltip_text(_('Edit the tag list'))
         button.connect('button-press-event', self.cb_edit)
